@@ -10,7 +10,7 @@ use Zim::Components::TreeView;
 use Zim::Components::PathBar;
 use Zim::Components::PageView;
 
-our $VERSION = 0.06;
+our $VERSION = 0.07;
 our $LONG_VERSION = << "EOT";
 zim $VERSION
 
@@ -97,6 +97,8 @@ sub new {
 	my ($class, $settings) = @_;
 	
 	%$settings = (%CONFIG, %$settings); # set defaults
+	$$settings{root} ||= $$settings{default_root};
+	$$settings{home} ||= $$settings{default_home};
 	my $self = bless {settings => $settings}, $class;
 	$self->load_config;
 
@@ -167,7 +169,7 @@ sub gui_init {
 	$vbox->add($hpaned);
 	$self->{hpaned} = $hpaned;
 
-	my $r_vbox = Gtk2::VBox->new(0, 3);
+	my $r_vbox = Gtk2::VBox->new(0, 5);
 	$hpaned->add2($r_vbox);
 	$self->{r_vbox} = $r_vbox;
 
@@ -188,7 +190,7 @@ sub gui_init {
 	$toolbar->append_space;
 	
 	$self->add_button('Home', 'Alt Home', 'gtk-home',
-		sub {$self->go(':'.$self->{settings}{home})} );
+		sub {$self->load_page($self->{settings}{home})} );
 	my $back_button = $self->add_button(
 		'Go Back', 'Alt Left', 'gtk-go-back', sub {$self->go_back(1)} );
 	my $forw_button = $self->add_button(
@@ -225,7 +227,7 @@ sub gui_init {
 
 	#my $actions_button = add_button(
 	#	'Actions', '', 'gtk-execute', sub {});
-	$self->add_button('About', 'F1', 'gtk-dialog-info',
+	$self->add_button('About', 'F1', 'gtk-help',
 		sub {$self->about_dialog} );
 
 
@@ -239,11 +241,11 @@ sub gui_init {
 		my $namespace = $self->{page}->namespace;
 		$namespace = $self->{settings}{home} if $namespace eq ':';
 		$namespace =~ s/^:?/:/;
-		$self->go($namespace);
+		$self->load_page($namespace);
 	} );
 	$self->add_key('Alt D', sub {
 		my $page = strftime($self->{settings}{date_page}, localtime);
-		$self->go($page);
+		$self->load_page($page);
 	} );
 	$self->add_key('Ctrl  ', sub { # Ctrl-Space
 		if ($self->{settings}{pane_vis}) {
@@ -254,7 +256,7 @@ sub gui_init {
 	} );
 	$self->add_key('Alt J', sub {$self->toggle_path_bar} );
 
-	$self->signal_connect('history_changed', sub {
+	$self->signal_connect('page_loaded', sub {
 		my ($back, $forw) = $history->state;
 		$back_button->set_sensitive($back?1:0);
 		$forw_button->set_sensitive($forw?1:0);
@@ -376,6 +378,12 @@ This method runs the GUI application.
 sub main_loop {
 	my $self = shift;
 	
+	unless ($self->{page}) { # FIXME what if we are read_only and page does not exist ?
+		my $page = $self->{objects}{History}->get_current
+			|| $self->{settings}{home};
+		$self->load_page($page);
+	}
+	
 	$self->{message_lock} = 1;
 	$self->toggle_pane($self->{settings}{pane_vis});
 	$self->toggle_path_bar($self->{settings}{pathbar_type});
@@ -394,7 +402,7 @@ Exit the main loop.
 
 sub quit {
 	my $self = shift;
-	&on_delete_event($self->{window}, $self);
+	&on_delete_event($self->{window}, $self) && return;
 	Gtk2->main_quit;
 }
 
@@ -403,7 +411,8 @@ sub on_delete_event { # Called when the window is deleted
 	my $window = shift;
 	@{$self->{settings}}{'width', 'height'} = $window->get_size;
 	
-	$self->save_page;
+	$self->{save_lock} = 0; # just to be sure
+	$self->save_page() || return 1;
 	$self->save_config;
 
 	unless ($self->{settings}{read_only}) {
@@ -415,16 +424,17 @@ sub on_delete_event { # Called when the window is deleted
 	return 0; # let window continu with destruction
 }
 
-=item C<go(NAME)>
+=item C<link_clicked(LINK)>
 
-Loads a new page adding it to the history or opens an external
-uri in a browser. NAME is considered to be either an url or a page name.
+Loads a page in zim or open an external uri in a browser.
+LINK is considered to be either an url or a page name.
 Page names are resolved as relative links first.
 
 =cut
 
-sub go {
+sub link_clicked {
 	my ($self, $link) = @_;
+	$link = $$link[1] if ref $link; # link_data = [bit, link]
 	return warn "Warning: You tried to folow an empty link.\n"
 		unless length $link;
 
@@ -440,20 +450,11 @@ sub go {
 			exec $browser;
 			exit 1; # just to be sure
 		}
-		return;
 	}
-
-	$link = $self->{page}->resolve_link($link);
-	
-	return $self->error_dialog("Page does not exist:\n$link")
-		if $self->{settings}{read_only}
-		and ! $self->{repository}->page_exists($link);
-
-	my $hist = $self->{objects}{History};
-	$hist->current; # make sure the hist record is created
-	$self->load_page($link);
-	$hist->push($self->{page});
-	$self->signal_emit('history_changed');
+	else {
+		$link = $self->{page}->resolve_link($link);
+		$self->load_page($link);
+	}
 }
 
 =item C<go_back(INT)>
@@ -467,8 +468,7 @@ sub go_back {
 	my $i = shift || 1;
 	my $hist = $self->{objects}{History};
 	$hist->back($i) or return;
-	$self->load_page($hist->current);
-	$self->signal_emit('history_changed');
+	$self->load_page($hist->get_current);
 }
 
 =item C<go_forw(INT)>
@@ -482,38 +482,77 @@ sub go_forw {
 	my $i = shift || 1;
 	my $hist = $self->{objects}{History};
 	$hist->forw($i) or return;
-	$self->load_page($hist->current);
-	$self->signal_emit('history_changed');
+	$self->load_page($hist->get_current);
 }
 
+=item C<reload()>
+
+Save and reload the current page.
+
+=cut
+
 sub reload {
-	$_[0]->load_page($_[0]->{objects}{History}->current);
+	my $self = shift;
+	my $hist = $self->{objects}{History};
+	$self->load_page($hist->get_current);
 }
 
 =item C<load_page(PAGE)>
 
-Method used by C<go()> and friends to actually load a new page.
-Wrapper for C<Zim::Repository->load_page()>.
+Loads a new page in the PageView, updates history etc. when
+necessary. PAGE should be either an absolute page name, a history record
+or a page object.
 
 =cut
 
 sub load_page {
 	my ($self, $page) = @_;
 	
-	$self->save_page;
-
-	$page = $page->{name} if ref($page) eq 'HASH'; # hist record
-	if (ref $page) { $self->{page} = $page } # page object
-	else {
-		$page =~ s/^:+|:+$//g; # FIXME ??
-		$self->{page} = $self->{repository}->load_page($page);
+	if ($self->{page}) {
+		$self->{save_lock} = 0; # just to be sure
+		$self->save_page || return;
 	}
 
-	$self->{objects}{PageView}->load_page($self->{page});
+	my $rec;
+	if (ref($page) eq 'HASH') { # hist record
+		($rec, $page) = ($page, $page->{name});
+	}
+	
+	if (ref $page) { # page object
+		$self->{page} = $page;
+		$page = $self->{page}->name;
+	}
+	else {
+		return warn "Warning: You tried to load an empty name.\n"
+			unless length $page;
+		
+		my $p;
+		eval { $p = $self->{repository}->load_page($page) };
+		return $self->error_dialog("Could not load page $page\n\n$@")
+			if $@;
+		
+		return $self->error_dialog("Page does not exist:\n$page")
+			if $self->{settings}{read_only}
+			and $p->status eq 'new';
+
+		$self->{page} = $p;
+	}
+
+	$self->{objects}{History}->set_current($self->{page});
+	eval { $self->{objects}{PageView}->load_page($self->{page}) };
+	if ($@) {
+		$self->exit_error("Could not load page $page\n\n$@");
+		# TODO handle more gratiously
+		# .. switching read_only would help here
+	}
+	$rec = $self->{objects}{History}->record($page) unless defined $rec;
+	$self->{objects}{PageView}->set_state(%$rec) if defined $rec;
 	$self->signal_emit('page_loaded', $self->{page}->realname);
 	
 	$self->{window}->set_title($self->{page}->name.' - Zim');
 	$self->update_status();
+
+	$self->{objects}{PageView}->grab_focus;
 }
 
 =item C<save_page(FORCE)>
@@ -523,20 +562,29 @@ whether it was modified or not. Wrapper for C<Zim::Repository->save_page()>.
 
 =cut
 
+# TODO split out to save_page_if_modified() and save_page()
+# save_page_if_modified(FORCE) overrules save_lock
+# save_page() just saves it, no questions asked
+# (maybe call the first one check_save_page() ?)
+
 sub save_page {
 	my ($self, $force) = @_;
 	my $modified = $self->{objects}{PageView}->modified;
 	return 0 if $self->{app}{settings}{read_only};
-	return 1 unless $force or $modified and ! $self->{save_lock};
+	unless ($force) {
+		return 1 if ! $modified;
+		return 0 if $self->{save_lock};
+	}
 	
 	$self->{save_lock} = 1; # Used to block autosave while "could not save" dialog
 	
 	my $page = $self->{page};
-	my $content = $self->{objects}{PageView}->save_page($page);
+	$self->{objects}{PageView}->save_page($page);
+	my $tree = $page->parse_tree;
 	
 	eval {
 		my $old_status = $page->status;
-		if ($content) {
+		if (@$tree > 2) { # content
 			$self->{repository}->save_page($page);
 			$self->signal_emit('page_created', $page->realname)
 				if $old_status eq 'new' or $old_status eq 'deleted';
@@ -554,15 +602,93 @@ sub save_page {
 	else { # FIXME this dialog could be a lot better
 		my $page = $self->prompt_page_dialog($page->name,
 			'Save As', 'Save Page As:', $@, 'gtk-dialog-error' );
-		if ($page =~ /\S/) {
-			# FIXME check page exists
-			$page =~ s/^:+|:+$//g;
-			$self->{page} = $self->{repository}->load_page($page);
-			$self->save_page('FORCE'); # recurs
-		}
+		return 0 unless defined $page;
+			# leave lock in position after returning
+		# FIXME check page exists
+		$self->{page} = $self->{repository}->load_page($page);
+		$self->save_page('FORCE'); # recurs
 	}
 
 	$self->{save_lock} = 0;
+	return 1;
+}
+
+=item C<move_page(FROM, TO)>
+
+Wrapper for C<Zim::Repository->move_page()>.
+
+Move page from FROM to TO. If TO is undefined a popup is 
+shown to ask for a page name.
+
+=cut
+
+sub move_page {
+	my ($self, $from, $to) = @_;
+
+	$from = $self->{page}->realname unless defined $from;
+	
+	my $move_current = ($from eq $self->{page}->realname);
+	if ($move_current) { # FIXME clearer interface for this
+		$self->{save_lock} = 0 if $move_current;
+		$self->save_page or return;
+	}
+	else { $self->save_page }
+	
+	$to = $self->prompt_move_page_dialog($from)
+		unless defined $to;
+	
+	return unless defined $to;
+
+	# TODO check existence of target file
+	#print "Move '$from' to '$to'\n";
+	eval { $self->{repository}->move_page($from, $to) };
+	return $self->error_dialog("Could not move $from to $to\n\n$@") if $@;
+
+	# TODO trigger this with signals
+	# problem is that we might have moved a whole namespace
+	$self->{objects}{TreeView}->{_loaded} = 0;
+	$self->{objects}{TreeView}->load_index;
+
+	$self->load_page($to) if $move_current;
+}
+
+=item C<delete_page(PAGE)>
+
+Wrapper for C<Zim::Repository->delete_page>.
+Asks the user for confirmation.
+
+If PAGE is undefined the current page is deleted.
+
+=cut
+
+sub delete_page {
+	# FIXME no option to delete a whole namespace so far
+	# although probably what you wnat, this is not consistent
+	# with move_page()
+	# FIXME integrate with the save_page() logic, now we have 
+	# redundant code here
+	my ($self, $page) = @_;
+	$page = $self->{page}->realname unless defined $page;
+	
+	my $name = ref($page) ? $page->realname : $page;
+	my $delete_current = ($name eq $self->{page}->realname);
+	$self->save_page; # Mind you, overriding save lock could lock us here
+	
+	return unless $self->prompt_confirm_delete_page($name);
+	
+	$page = $self->{page} if $delete_current; # make sure we have the right object
+	eval { $self->{repository}->delete_page($page) };
+	return $self->error_dialog("Could not delete $name\n\n$@") if $@;
+
+	$self->signal_emit('page_deleted', $name);
+	if ($delete_current) {
+		eval { $self->{objects}{PageView}->load_page($page) };
+		if ($@) {
+			$self->exit_error("Could not load page $name\n\n$@");
+			# TODO handle more gratiously
+			# .. switching read_only would help here
+		}
+	}
 }
 
 =item C<message(STRING)>
@@ -731,7 +857,6 @@ sub toggle_path_bar {
 	if ($type eq 'trace' or $type eq 'namespace') {
 		$path_bar->widget->show_all;
 		$path_bar->set_type($type);
-		$path_bar->on_history_changed();
 	}
 	elsif ($type eq 'hidden') {
 		$path_bar->widget->hide_all;
@@ -745,6 +870,8 @@ sub toggle_path_bar {
 
 
 # TODO the various dialogs could be out-sourced using Autoloader
+# also a lot of code could be reduced by haveing a generic
+# _prompt_dialog(title => ... text => ... fields => { })
 
 =item C<exit_error(ERROR)>
 
@@ -785,7 +912,7 @@ sub goto_page_dialog {
 	my $self = shift;
 	my $page = $self->{page}->namespace;
 	$page = $self->prompt_page_dialog($page, 'Go to', '_Go to Page:');
-	$self->go($page) if $page =~ /\S/;
+	$self->link_clicked($page) if defined $page;
 }
 
 =item C<promt_page_dialog(PAGE, TITLE, PROMPT, TEXT, ICON)>
@@ -841,6 +968,41 @@ sub prompt_page_dialog {
 	else                      { $page = undef            }
 	$dialog->destroy;
 
+	$page = undef unless $page =~ /\S/;
+	return $page;
+}
+
+sub prompt_move_page_dialog {
+	my ($self, $page) = @_;
+	
+	my $title = "Move page";
+	my $dialog = Gtk2::Dialog->new(
+		$title, $self->{window},
+	       	[qw/modal destroy-with-parent no-separator/],
+		'gtk-cancel' => 'cancel',
+	);
+	$dialog->set_resizable(0);
+	$dialog->set_border_width(5);
+	$dialog->set_icon($self->{window}->get_icon);
+	$dialog->set_default_response('ok');
+
+	my $move_button = $self->new_button('gtk-save', '_Move');
+	$dialog->add_action_widget($move_button, 'ok');
+	# $dialog->set_default_response('ok'); FIXME
+
+	my $entry = Gtk2::Entry->new();
+	$entry->signal_connect(activate => sub { $dialog->response('ok') });
+	$entry->set_text($page);
+	$dialog->vbox->add($entry);
+
+	$dialog->show_all;
+	$entry->select_region(length($1), -1) if $page =~ /^(.+:)/;
+
+	if ($dialog->run eq 'ok') { $page = $entry->get_text }
+	else                      { $page = undef            }
+	$dialog->destroy;
+
+	$page = undef unless $page =~ /\S/;
 	return $page;
 }
 
@@ -895,6 +1057,8 @@ sub prompt_link_dialog {
 		$text = $text_entry->get_text;
 		$link = $link_entry->get_text;
 		$dialog->destroy;
+		return undef
+			unless $text =~ /\S/ or $link =~ /\S/;
 		return $text, $link;
 	}
 	else {
@@ -903,6 +1067,39 @@ sub prompt_link_dialog {
 	}
 }
 
+=item C<prompt_confirm_delete_page(PAGE)>
+
+Ask the user if they are sure about deleting a page.
+Returns true if the user wants to proceed.
+
+=cut
+
+sub prompt_confirm_delete_page {
+	my ($self, $page) = @_;
+	
+	my $title = "Delete page";
+	my $dialog = Gtk2::Dialog->new(
+		$title, $self->{window},
+	       	[qw/modal destroy-with-parent no-separator/],
+		'gtk-cancel'  => 'cancel',
+		'gtk-ok'      => 'ok'
+	);
+	$dialog->set_resizable(0);
+	$dialog->set_border_width(5);
+	$dialog->set_icon($self->{window}->get_icon);
+	$dialog->set_default_response('ok');
+
+	$dialog->vbox->add( Gtk2::Label->new( << "EOT" ) );
+Are you sure you want to
+delete $page ?
+EOT
+	$dialog->show_all;
+	my $ok = ($dialog->run eq 'ok') ? 1 : 0;
+	$dialog->destroy;
+	
+	return $ok;
+}
+
 =item C<about_dialog()>
 
 This dialog tells you about the version of zim you are using,
@@ -957,6 +1154,92 @@ sub about_dialog {
 	$dialog->destroy;
 }
 
+=item C<filechooser_dialog(FILE, TITLE)>
+
+Ask the user for a filename. FILE is the suggested filename.
+
+=cut
+
+sub filechooser_dialog {
+	my ($self, $file, $title) = @_;
+	
+	my $dialog;
+	$title ||= 'Select File';
+	if (Gtk2->CHECK_VERSION(2, 4, 0) and $Gtk2::VERSION >= 1.040) {
+		$dialog = Gtk2::FileChooserDialog->new(
+			$title, $self->{window}, 'open',
+			'gtk-cancel' => 'cancel',
+			'gtk-ok'     => 'ok'
+		);
+	}
+	else { # old & ugly interface
+		$dialog = Gtk2::FileSelection->new($title);
+	}
+	$dialog->set_icon($self->{window}->get_icon) if $self->{window};
+	$dialog->set_filename(File::Spec->rel2abs($file)) if defined $file;
+	$dialog->signal_connect('response', sub {
+		$file = $_[1] eq 'ok' ? $dialog->get_filename : undef;
+		$dialog->destroy;
+	} );
+	$dialog->run;
+
+	return $file;
+}
+
+sub prompt_root_dialog {
+	my ($self, $page, $title, $prompt, $text, $icon) = @_;
+
+	my $dialog = Gtk2::Dialog->new(
+		"Open directory - Zim", undef,
+	       	[qw/modal destroy-with-parent no-separator/],
+		'gtk-cancel' => 'cancel',
+		'gtk-open'   => 'ok',
+	);
+	$dialog->set_resizable(0);
+	$dialog->set_border_width(5);
+	$dialog->set_icon(
+		Gtk2::Gdk::Pixbuf->new_from_file($self->{settings}{icon_file}) );
+	$dialog->set_default_response('ok');
+	
+	$dialog->vbox->add(
+		Gtk2::Image->new_from_file($self->{settings}{icon_file}) );
+
+	$dialog->vbox->add( Gtk2::Label->new( << 'EOT' ) );
+Welcom to Zim.
+
+Zim needs a directory to store its wiki page.
+Please enter a default directory below.
+
+You can open other directories by giving them as commandline
+a argument to zim(1).
+
+If you want to change the default directory later you need to
+edit the "default_root" option in the config file.
+(TODO: add a nice dialog for changing config options)
+EOT
+	
+	my $hbox  = Gtk2::HBox->new(0, 5);
+	$hbox->set_border_width(5);
+	$dialog->vbox->add($hbox);
+	
+	my $label = Gtk2::Label->new("Open directory: ");
+	$hbox->add($label);
+	
+	my $entry = Gtk2::Entry->new();
+	$entry->signal_connect(activate => sub { $dialog->response('ok') });
+	$entry->set_text("$ENV{HOME}/zim_pages/");
+	$hbox->add($entry);
+	
+	my $root;
+	$dialog->show_all;
+	if ($dialog->run eq 'ok') { $root = $entry->get_text }
+	else                      { $root = undef            }
+	$dialog->destroy;
+
+	$root = undef unless $root =~ /\S/;
+	return $root;
+}
+
 # "Search" dialog
 #
 # This dialog asks for a search query and then generates
