@@ -3,11 +3,15 @@ package Zim::Components::PageView;
 use strict;
 use vars '$AUTOLOAD';
 use Gtk2;
-use Gtk2::Pango;             # pango constants
-use Gtk2::Ex::HyperTextView; # custom widget
+#use Gtk2::Spell;
+use Gtk2::Pango;               # pango constants
+use Gtk2::Ex::HyperTextView;   # custom widget
+use Gtk2::Ex::HyperTextBuffer; #    "     "
 use POSIX qw(strftime);
 
-our $VERSION = 0.01;
+our $VERSION = 0.07;
+
+# TODO move more logic into Gtk2::Ex::HyperTextBuffer !
 
 =head1 NAME
 
@@ -31,20 +35,6 @@ Undefined methods are AUTOLOADED to the Gtk2::Ex::HyperTextView object.
 my ($k_tab, $k_return, $k_kp_enter, $k_backspace, $k_escape, $k_multiply) =
 	@Gtk2::Gdk::Keysyms{qw/Tab Return KP_Enter BackSpace Escape KP_Multiply/};
 
-# It seems to me that 10 * PANGO_SCALE is the normal font size
-# TODO make tag styles configable
-our %TAGS = (
-	B     => [weight => PANGO_WEIGHT_BOLD],
-	I     => [style => 'italic'],
-	U     => [background => 'yellow'],
-	C     => [family => 'monospace', wrap_mode => 'none'],
-	head1 => [weight => PANGO_WEIGHT_BOLD, size => 10*1.2**3 * PANGO_SCALE ],
-	head2 => [weight => PANGO_WEIGHT_BOLD, size => 10*1.2**2 * PANGO_SCALE ],
-	head3 => [weight => PANGO_WEIGHT_BOLD, size => 12 * PANGO_SCALE        ],
-	head4 => [weight => PANGO_WEIGHT_BOLD, size => 10 * PANGO_SCALE        ],
-	head5 => [style  => 'italic', weight => PANGO_WEIGHT_BOLD, size => 10 * PANGO_SCALE],
-);
-
 our %UNDO_STEPS = (
 	delete     => 'insert',
 	insert     => 'delete',
@@ -52,6 +42,22 @@ our %UNDO_STEPS = (
 	remove_tag => 'apply_tag'
 );
 
+our %TAG_ALIASES = (
+	B         => 'bold',
+	C         => 'verbatim',
+	I         => 'italic',
+	L         => 'link',
+	U         => 'underline',
+	bold      => 'B',
+	verbatim  => 'C',
+	italic    => 'I',
+	link      => 'L',
+	underline => 'U',
+);
+
+$Gtk2::Ex::HyperTextBuffer::TAGS{link} = [foreground => 'blue'];
+$Gtk2::Ex::HyperTextBuffer::TAGS{underline} = [background => 'yellow'];
+
 sub new {
 	my $class = shift;
 	my $self = bless {@_}, $class;
@@ -82,7 +88,7 @@ sub init { # called by new()
 		1, 0, 'left' => 40 * PANGO_SCALE ) );
 	$htext->{link_properties} = [foreground => 'blue']; # TextTag properties
 	$htext->signal_connect(link_clicked =>
-		sub { $self->{app}->go($_[1][1])                          }  );
+		sub { $self->{app}->link_clicked($_[1][1])                }  );
 	$htext->signal_connect(link_enter =>
 		sub { $self->{app}->push_status("Go to $_[1][1]", 'link') }  );
 	$htext->signal_connect(link_leave =>
@@ -93,6 +99,9 @@ sub init { # called by new()
 	$scroll_window->add($htext);
 	$self->{htext} = $htext;
 
+	#my $spell = Gtk2::Spell->new_attach($htext);
+	#$self->{spell} = $spell;
+
 	# init search box
 	my $hbox = Gtk2::HBox->new(0, 5);
 	$vbox->pack_start($hbox, 0, 1, 0);
@@ -123,16 +132,16 @@ sub init { # called by new()
 		my $style_menu = Gtk2::Menu->new;
 		$style_menu->append( Gtk2::TearoffMenuItem->new() );
 		for my $tag (
-			[ 'Head 1',   'Ctrl 1' => 'head1'  ],
-			[ 'Head 2',   'Ctrl 2' => 'head2'  ],
-			[ 'Head 3',   'Ctrl 3' => 'head3'  ],
-			[ 'Head 4',   'Ctrl 4' => 'head4'  ],
-			[ 'Head 5',   'Ctrl 5' => 'head5'  ],
-			[ 'Normal',   'Ctrl 6' => 'Normal' ],
-			[ 'Bold',     'Ctrl B' => 'B'      ],
-			[ 'Italic',   'Ctrl I' => 'I'      ],
-			[ 'Mark',     'Ctrl U' => 'U'      ],
-			[ 'Verbatim', 'Ctrl T' => 'C'      ],
+			[ 'Head 1',   'Ctrl 1' => 'head1'     ],
+			[ 'Head 2',   'Ctrl 2' => 'head2'     ],
+			[ 'Head 3',   'Ctrl 3' => 'head3'     ],
+			[ 'Head 4',   'Ctrl 4' => 'head4'     ],
+			[ 'Head 5',   'Ctrl 5' => 'head5'     ],
+			[ 'Normal',   'Ctrl 6' => 'Normal'    ],
+			[ 'Bold',     'Ctrl B' => 'bold'      ],
+			[ 'Italic',   'Ctrl I' => 'italic'    ],
+			[ 'Mark',     'Ctrl U' => 'underline' ],
+			[ 'Verbatim', 'Ctrl T' => 'verbatim'  ],
 		) {
 			my $code = sub { $self->apply_tag($$tag[2]) };
 		
@@ -178,6 +187,9 @@ sub init { # called by new()
 	$self->{app}->add_button(
 		'Edit Link', 'Ctrl E', 'gtk-properties',
 		sub {$self->edit_link_dialog} ) unless $read_only;
+	#$self->{app}->add_button(
+	#	'Insert image', undef, 'gtk-select-color',
+	#	sub {$self->{app}->filechooser_dialog} ) unless $read_only;
 
 	$self->{app}->add_key('Alt /', sub {$self->show_search} );
 
@@ -244,10 +256,10 @@ sub find {
 	
 	my ($start, $end);
 	unless ($direction == -1) { # forward
-		($start, $end) = $iter->forward_search($string, 'visible-only');
+		($start, $end) = $iter->forward_search($string, []);
 		unless (defined $start) { # wrap around
 			$iter = $buffer->get_start_iter;
-			($start, $end) = $iter->forward_search($string, 'visible-only');
+			($start, $end) = $iter->forward_search($string, []);
 			return unless defined $start;
 		}
 	}
@@ -296,7 +308,8 @@ sub on_populate_popup { # add items to context menu
 	my ($htext, $menu) = @_;
 	my $buffer = $htext->get_buffer;
 	my $link = $htext->get_link_at_pointer();
-	return unless defined $link;
+	return unless $link;
+	$link = $$link[1];
 	
 	my $seperator = Gtk2::MenuItem->new();
 	$seperator->show;
@@ -314,21 +327,21 @@ sub on_populate_popup { # add items to context menu
 }
 
 sub on_key_press_event { # some extra keybindings
+	# FIXME for more consistent behaviour test for selections
 	my ($htext, $event, $self) = @_;
 	my $val = $event->keyval;
 
-	#if ($self->{app}{settings}{read_only}) {
-		#return 0 unless $val == ord('/');
-		#$self->{hbox}->show;
-		#$self->{entry}->grab_focus;
-		#return 1;
-	#}
+	if ($self->{app}{settings}{read_only}) {
+		return 0 unless $val == ord('/');
+		$self->show_search;
+		return 1;
+	}
 	
 	if ($val == $k_return or $val == $k_kp_enter) { # Enter
 		my $buffer = $htext->get_buffer;
 		my $iter = $buffer->get_iter_at_mark($buffer->get_insert());
 		return 1 if defined $htext->click_if_link_at_iter($iter);
-		$self->parse_line($iter) or $buffer->insert_at_cursor("\n");
+		$self->parse_line($iter) or return 0;
 		$htext->scroll_mark_onscreen( $buffer->get_insert );
 		return 1;
 	}
@@ -387,6 +400,27 @@ sub get_state {
 	return cursor => $cursor, undo => $self->{undo}, redo => $self->{redo};
 }
 
+=item C<set_state(property => value, ..)>
+
+Set a number of properties that could be saved in the history.
+
+=cut
+
+sub set_state {
+	my $self = shift;
+	my %rec = @_;
+	
+	if (defined $rec{cursor}) {
+		$self->{buffer}->place_cursor(
+			$self->{buffer}->get_iter_at_offset($rec{cursor}) );
+
+		$self->{htext}->scroll_mark_onscreen( $self->{buffer}->get_insert );
+	}
+	
+	$self->{undo} = $rec{undo} || [];
+	$self->{redo} = $rec{redo} || [];
+}
+
 =item C<get_status()>
 
 Returns an info string for the current buffer.
@@ -410,11 +444,12 @@ sub load_page {
 	my ($self, $page) = @_;
 	
 	# create a new TextBuffer
-	my $buffer = Gtk2::TextBuffer->new();
-	$buffer->create_tag($_ => @{$TAGS{$_}}) for keys %TAGS;
+	my $buffer = Gtk2::Ex::HyperTextBuffer->new();
+	$buffer->tag_aliases(\%TAG_ALIASES);
 	$self->{buffer} = $buffer;
 	$self->{htext}->set_buffer($buffer);
-	$self->load_parsetree($page);
+	#$self->load_parsetree($page);
+	$buffer->set_parse_tree( $page->parse_tree );
 	unless ($self->{app}{settings}{read_only}) {
 		# connect signals _after_ load_parsetree()
 		$buffer->signal_connect(delete_range => \&on_delete_range, $self);
@@ -426,79 +461,14 @@ sub load_page {
 	}
 	$buffer->set_modified(0);
 
-	# look for previous cursor position and undo stack
-	my $rec = $self->{app}->History->record($page);
-	if (defined $rec) {
-		$buffer->place_cursor(
-			$buffer->get_iter_at_offset($rec->{cursor}) );
-
-		$self->{undo} = $rec->{undo};
-		$self->{redo} = $rec->{redo};
-	}
-	else {
-		$buffer->place_cursor(
-			$buffer->get_iter_at_offset(0) )
-				unless $page->status eq 'new';
-		
-		$self->{undo} = [];
-		$self->{redo} = [];
-	}
-
+	$buffer->place_cursor(
+		$buffer->get_iter_at_offset(0) )
+			unless $page->status eq 'new';
+	
+	$self->{undo} = [];
+	$self->{redo} = [];
+	
 	$self->{htext}->scroll_mark_onscreen( $buffer->get_insert );
-	$self->{htext}->grab_focus;
-}
-
-=item C<load_parsetree()>
-
-This method parses the data from the current page object and
-puts it into the text buffer
-
-=cut
-
-sub load_parsetree {
-	my ($self, $page) = @_;
-	#use Data::Dumper; print Dumper $page;
-	my $tree = $page->parse_tree;
-	$self->insert_block($_) for @{$tree}[2 .. $#$tree];
-}
-
-sub insert_block {
-	my ($self, $block) = @_;
-	my $buffer = $self->{buffer};
-	unless (ref $block) {
-		s/^(\s*)[\*\xB7](\s+)/$1\x{2022}$2/mg; # bullets
-		# \xB7 is the latin1 "high dot" 
-		# \x2022 is the utf8 bullet
-		# FIXME lists should be identified by the backend
-		$buffer->insert_at_cursor($_);
-	}
-	else {
-		my ($tag, $meta, @node) = @$block;
-		my $iter = $buffer->get_iter_at_mark(
-				$buffer->get_insert() );
-		if ($tag eq 'Para') {
-			$self->insert_block($_) for @node; # recurs
-		}
-		elsif ($tag eq 'L') {
-			my ($to, $text) = ($meta->{to}, $node[0]);
-			my $link = [($to eq $text), $to];
-			$self->{htext}->insert_link_at_iter($iter, $text, $link);
-		}
-		elsif ($tag eq 'image') { # experimental feature
-			my $file = File::Spec->catfile(
-				$self->{app}{settings}{data_dir}, $meta->{src});
-			$file = Gtk2::Gdk::Pixbuf->new_from_file($file);
-			$buffer->insert_pixbuf($iter, $file);
-		}
-		else {
-			if ($tag eq 'Verbatim') {
-				$node[0] =~ s/^/\t/mg;
-				$tag = 'C';
-			}
-			$buffer->insert_with_tags_by_name(
-				$iter, $node[0], $tag );
-		}
-	}
 }
 
 =item C<modified()>
@@ -526,67 +496,7 @@ Returns true if the buffer is not empty.
 
 sub save_page {
 	my ($self, $page) = @_;	
-	return $self->save_parsetree($page);
-}
-
-=item C<save_parsetree(PAGE)>
-
-This routine gets the text from the buffer and puts it back 
-into the current page object.
-
-=cut
-
-sub save_parsetree {
-	my ($self, $page) = @_;
-	my $buffer = $self->{buffer};
-	my ($start, $end) = $buffer->get_bounds;
-	my $text = $buffer->get_text($start, $end, 0); # start, end, hidden_chars
-
-	return 0 unless $text =~ /\S/;
-	
-	$page->clear;
-	my $last = length $text;
-	for (reverse $self->find_tags) {
-		my ($start, $end, $name, $meta) = @$_;
-		next unless $start < $last; # just to be sure
-		my $rest = substr($text, $end, ($last-$end), '');
-		my $block = substr($text, $start, ($end-$start), '');
-		#print "$name from $start till $end : $block\n";
-		$rest =~ s/\x{2022}/\*/g;
-		if ($name eq 'C' and $block =~ /\n/) {
-			$name = 'Verbatim';
-			$block =~ s/^\t//mg;
-		}
-		$page->unshift_blocks([$name, $meta, $block], $rest);
-		$last = $start;
-	}
-	$page->unshift_blocks($text) if length $text;
-	#use Data::Dumper; print Dumper $page->parse_tree;
-	$page->clean;
-	
-	return 1;
-}
-
-sub find_tags { # Method to find all tags in the buffer with their ranges
-	my $self = shift;
-	my $buffer = $self->{buffer};
-	my $table  = $buffer->get_tag_table;
-	my (@tags, @tag_objs);
-#	print "table size: ", $table->get_size, "\n";
-	$table->foreach(sub { push @tag_objs, @_ });
-	for my $tag (@tag_objs) { # iterate over named and anonymous tags
-		my $iter = $buffer->get_start_iter;
-		while ( $iter->begins_tag($tag) or $iter->forward_to_tag_toggle($tag) ) {
-			my @tag = ( $iter->get_offset );
-			$iter->forward_to_tag_toggle($tag) or last;
-			push @tag, $iter->get_offset;
-			if ($tag->{is_link}) { push @tag, 'L', {to => $tag->{link_data}[1]} }
-			else                 { push @tag, $tag->get_property('name'), {} }
-			push @tags, \@tag;
-			last if $iter->is_end; # prevent infinite loop
-		}
-	}
-	return sort {$$a[0] <=> $$b[0]} @tags;
+	$page->parse_tree( $self->{buffer}->get_parse_tree );
 }
 
 =item C<parse_backspace(ITER)>
@@ -658,7 +568,8 @@ makes it never reaching the widget.
 
 sub parse_word {
 	my ($self, $iter, $char) = @_;
-	return unless $char eq ' ' or $char eq "\t";
+	return 0 unless $char eq ' ' or $char eq "\t";
+	return 0 if $self->_is_verbatim;
 	my $buffer = $self->{buffer};
 	my $lf = $buffer->get_iter_at_line( $iter->get_line );
 	my $line = $buffer->get_text($lf, $iter, 0) . $char;
@@ -684,6 +595,16 @@ sub parse_word {
 	return 0;
 }
 
+sub _is_verbatim {
+	my $self = shift;
+	my $buffer = $self->{buffer};
+	my $iter = $buffer->get_iter_at_mark( $buffer->get_insert );
+	for ($iter->get_tags) {
+		return 1 if lc($_->get_property('name')) eq 'verbatim';
+	}
+	return 0;
+}
+
 =item C<apply_tag(TAG)>
 
 Applies the tag with the name TAG to any selected text.
@@ -692,10 +613,13 @@ Applies the tag with the name TAG to any selected text.
 
 sub apply_tag {
 	my ($self, $tag) = @_;
-	my ($start, $end) = $self->get_selection($tag);
+	my ($start, $end) = $self->get_selection;
 	return unless defined $start;
 
-	# TODO what if selection contains linebreaks ??
+	if ($start->get_line != $end->get_line) {
+		if ($tag eq 'verbatim') { $tag = 'Verbatim' }
+		# TODO what if selection contains linebreaks ??
+	}
 	
 	my $buffer = $self->{buffer};
 	$buffer->remove_all_tags($start, $end);
@@ -709,19 +633,13 @@ sub apply_tag {
 		$buffer->insert($buffer->get_iter_at_offset($end+1), "\n")
 			unless ! defined $end;
 	}
-	elsif ($tag eq 'C') {
-		my $text = $buffer->get_text($start, $end, 1);
-		($start, $end) = $self->selection_tab($start, $end)
-			if $text =~ /\n/ and $text =~ /^[^\t]/m;
-		$buffer->remove_all_tags($start, $end);
-		$buffer->apply_tag_by_name('C', $start, $end);
-	}
 }
 
 sub get_selection { # selects current word if no selections
 	my ($self, $tag) = @_;
 	my $buffer = $self->{buffer};
 	my ($start, $end) = $buffer->get_selection_bounds;
+#	print "selection: $start, $end\n";
 	if (!$end or $start == $end) {
 		# TODO autoselect word ( or line if tag =~ /^head/ );
 		return undef;
@@ -766,7 +684,7 @@ sub apply_link {
 		$buffer->set_modified(1);
 	}
 
-	$self->{app}->go($link)
+	$self->{app}->link_clicked($link)
 		if $self->{app}{settings}{read_only}
 		or $self->{app}{settings}{follow_new_link};
 }
@@ -788,7 +706,7 @@ sub toggle_bullets {
 		$text =~ s/^(\s*)[\*\x{2022}]\s+/$1/mg
 	}
 	else { # set bullets
-		$text =~ s/^(\s*)(?:\-\s+)?(\S)/$1\x{2022} $2/mg;
+		$text =~ s/^(\s*)(\S)/$1\x{2022} $2/mg;
 	}
 
 	$self->replace_selection($text, $start, $end);
@@ -827,7 +745,10 @@ sub selection_tab {
 	my $text = $buffer->get_text($start, $end, 1);
 	$text =~ s/^/\t/mg;
 
-	return $self->replace_selection($text, $start, $end);
+	my $verbatim = $self->_is_verbatim;
+	($start, $end) = $self->replace_selection($text, $start, $end);
+	$self->{buffer}->apply_tag_by_name('verbatim', $start, $end)
+		if $verbatim;
 }
 
 =item C<selection_backspace()>
@@ -842,13 +763,16 @@ sub selection_backspace {
 	return unless defined $start;
 	
 	my $text = $self->{buffer}->get_text($start, $end, 1);
+	my $verbatim = $self->_is_verbatim;
 	if ($text =~ s/^\t//mg) {
-		return $self->replace_selection($text, $start, $end);
-	}
-	else {
-		$self->{buffer}->delete($start, $end);
-		return undef;
+		($start, $end) = $self->replace_selection($text, $start, $end);
+		$self->{buffer}->apply_tag_by_name('Verbatim', $start, $end)
+			if $verbatim;
 	}
+	elsif ($verbatim) {
+		$self->{buffer}->remove_all_tags($start, $end);
+	}	
+	else { $self->replace_selection('', $start, $end) }
 }
 
 sub on_insert_text { # buffer, iter, string, length, self
