@@ -47,9 +47,11 @@ sub init { # called by new()
 	$self->{tree_store} = $tree_store;
 
 	my $tree_view = Gtk2::TreeView->new($tree_store);
-	my $cellrenderer = Gtk2::CellRendererText->new();
+	my $renderer = Gtk2::CellRendererText->new();
+	$renderer->signal_connect_swapped(edited => \&on_cell_edited, $self);
+	$self->{renderer} = $renderer;
 	my $page_column = Gtk2::TreeViewColumn->new_with_attributes(
-        	'Pages', $cellrenderer, 'text', 0);
+        	'Pages', $renderer, 'text', 0);
 	$tree_view->append_column($page_column);
 	$tree_view->set_headers_visible(0);
 	#$tree_view->set_reorderable(1);
@@ -57,6 +59,10 @@ sub init { # called by new()
 	$tree_view->signal_connect(key_press_event      => \&on_key_press_event);
 	$tree_view->signal_connect(button_release_event => \&on_button_release_event, $self);
 	$tree_view->signal_connect(row_activated        => \&on_row_activated, $self);
+	$tree_view->signal_connect(popup_menu           => sub {
+			my ($path) = $tree_view->get_cursor;
+			$self->popup_context_menu($path);
+		} );
 	$scroll_window->add($tree_view);
 	$self->{tree_view} = $tree_view;
 
@@ -136,32 +142,96 @@ sub on_key_press_event { # some extra keybindings for the treeview
 	return 1;
 }
 
-sub on_button_release_event { # single-click navigation
+sub on_button_release_event {
 	my ($tree_view, $event, $self) = @_;
-	return 0 if $event->type ne 'button-release' or $event->button != 1;
-
+	return 0 if $event->type ne 'button-release';
+	
 	my ($x, $y) = $event->get_coords;
 	my ($path, $column) = $tree_view->get_path_at_pos($x, $y);
 	return 0 unless $path;
-
-	my ($selected) = $tree_view->get_selection->get_selected_rows;
-	return 0 unless $selected and 0 == ($path->compare($selected));
-
-	my $model = $tree_view->get_model;
-	my $iter = $model->get_iter($path);
-	my (undef, $page) = $model->get($iter);
-	$tree_view->row_activated($path, $column)
-		unless $page eq $self->{app}{page}->realname;
+	
+	$self->activate_if_selected($path, $column)
+		if $event->button == 1;
+	$self->popup_context_menu($path, $event)
+		if $event->button == 3;
 	return 0;
 }
 
 sub on_row_activated { # load a page when clicked or entered
 	my $self = pop;
 	my ($tree_view, $path) = @_;
-	my $model = $tree_view->get_model;
+	my $page = $self->page_from_path($path);
+	$self->{app}->load_page($page)
+		unless $page eq $self->{app}{page}->realname;
+}
+
+=item C<page_from_path(PATH)>
+
+Returns the page name corresponding to a certain TreePath.
+
+=cut
+
+sub page_from_path {
+	my ($self, $path) = @_;
+	my $model = $self->{tree_view}->get_model;
 	my $iter = $model->get_iter($path);
-	my (undef, $page) = $model->get($iter);
-	$self->{app}->go(':'.$page);
+	my ($base, $real) = $model->get($iter);
+	return wantarray ? ($real, $base) : $real;
+}
+
+=item C<activate_if_selected(PATH, COLUMN)>
+
+Emit the "row_activated" signal if PATH was already selected.
+This method is used for the single-click navigation of the TreeView.
+
+COLUMN is an optional argument.
+
+=cut
+
+sub activate_if_selected { # single-click navigation
+	my ($self, $path, $column) = @_;
+	my $tree_view = $self->{tree_view};
+
+	my ($selected) = $tree_view->get_selection->get_selected_rows;
+	return 0 unless $selected and 0 == ($path->compare($selected));
+
+	my $page = $self->page_from_path($path);
+	$column ||= $tree_view->get_column(0);
+	$tree_view->row_activated($path, $column)
+		unless $page eq $self->{app}{page}->realname;
+}
+
+=item C<popup_context_menu(PATH, EVENT)>
+
+Show the context menu for TreePath PATH.
+
+EVENT is an optional argument.
+
+=cut
+
+sub popup_context_menu {
+	my ($self, $path, $event) = @_;
+	my $tree_view = $self->{tree_view};
+	my $page = $self->page_from_path($path);
+	
+	my $menu = Gtk2::Menu->new();
+	my $item1 = Gtk2::MenuItem->new('Move page');
+	$item1->signal_connect(
+		activate => sub { $self->{app}->move_page($page, undef, 1) });
+	$menu->append($item1);
+	my $item2 = Gtk2::MenuItem->new('Delete page');
+	$item2->signal_connect(
+		activate => sub { $self->{app}->delete_page($page) });
+	$menu->append($item2);
+	$menu->show_all;
+
+	my ($button, $time) = $event
+		? ($event->button, $event->time)
+		: (0, 0) ;
+	#my $pos_func = $event ? undef : sub {
+	#};
+	$menu->popup(undef, undef, undef, undef, $button, $time);
+	return 1; # needed for popup_menu signal (shift-F10), but why ?
 }
 
 # load_index()
@@ -239,9 +309,11 @@ sub unlist_page {
 	delete $$tree_iter{$page};
 }
 
-# select_page( page->realname )
-#
-# Update the tree_view to actually display and select a certain page.
+=item C<select_page(PAGE)>
+
+Update the TreeView to display and select a certain page.
+
+=cut
 
 sub select_page {
 	my ($self, $page) = @_;
