@@ -71,7 +71,7 @@ sub parse_verbatim { # parse pre formated paragraphs
 	return ['Verbatim', {}, $pre];
 }
 
-our @parser_subs = qw/parse_image parse_link parse_styles/;
+our @parser_subs = qw/parse_link parse_styles parse_image/;
 
 sub parse_para {
 	my ($class, $text) = @_;
@@ -142,6 +142,7 @@ Returns the text.
 =cut
 
 sub save_tree { ## unknown tags just fall through as plain text
+	# TODO add support for recursive tags
 	my ($class, $tree) = @_;
 	my ($name, $opt) = splice @$tree, 0, 2;
 	die "Invalid parse tree"
@@ -154,7 +155,7 @@ sub save_tree { ## unknown tags just fall through as plain text
 			$text .= $node;
 			next;
 		}
-		my ($tag, $tag_opt, @node) = @$node;
+		my ($tag, $meta, @node) = @$node;
 		if ($tag eq 'Para') {
 			$text .= $class->save_tree($node);
 		}
@@ -166,8 +167,11 @@ sub save_tree { ## unknown tags just fall through as plain text
 			my $n = 7 - $1;
 			$text .= ('='x$n)." $node[0] ".('='x$n);
 		}
+		elsif ($tag eq 'image') {
+			$text .= '{{'.$$meta{src}.'}}';
+		}
 		elsif ($tag eq 'L') {
-			my $to = $tag_opt->{to};
+			my $to = $$meta{to};
 			$to =~ s/^mailto://;
 			if ($to ne $node[0]) { $text .= "[[$to|$node[0]]]" }
 			elsif ($to =~ m#^\w+://#) { $text .= $to }
@@ -186,6 +190,8 @@ sub save_tree { ## unknown tags just fall through as plain text
 
 __END__
 
+=back
+
 =head1 AUTHOR
 
 Jaap Karssenberg (Pardus) E<lt>pardus@cpan.orgE<gt>
